####################################################################################################################
##																			##
## Program name: f11.def3 for f1<f2=f3=f4													##
## 		 f12.def3 for f1=f2<f3=f4													##
## 		 f13.def3 for f1=f2=f3<f4													##
## 																			##
## Purpose: To calculate the minimum f1 to have an specified chance of showing consistency by Definition 3.		##
##																			##
## Definition 3: Demonstrating that region effects exceed a proportion of the overall effect using hypothesis	##
##	 	 testing																##
##																			##
## Requirement: Need to install package "mvtnorm" in R.										##
##																			##
## Date: March 16, 2010																##
##																			##
## Author: Mingyu Li																##
##																			##
## Reference: Assessment of Consistency of Treatment Effects in Multi-Regional Clinical Trials, to appear in    	##
##            DIJ, 2010																##
##																			##
####################################################################################################################

## Load "mvtnorm"

library(mvtnorm)

# f = (x, (1-x)/3, (1-x)/3, (1-x)/3)

f11.def3 = function(alpha=0.025, beta=0.01, delta=0.005, sigma=0.013, u=c(1,1,1,1), pi=0, eta=0.4, set.pow=0.8)  

{
z.alpha = qnorm(1-alpha)

z.beta = qnorm(1-beta)

## Total number of patients for placebo group to detect overall treatment effect delta 
## with type I error=alpha, power=1-beta with 2 (trxt):1(placebo) allocation

N = 3*sigma^2*(z.alpha+z.beta)^2/2/delta^2	

N = round(N)

s = 4

un.pow=function(x, a)	# x=f[1] a=c(s, u, pi, delta, sigma, N, eta, set.pow)
	{
	s = a[1]
	u = a[2:(s+1)]
	pi = a[s+2]
	delta = a[s+3]
	sigma = a[s+4]
	N = a[s+5]
	eta = a[s+6]
	set.pow = a[s+7]

	z.eta = qnorm(1-eta)

	mean = (u-pi)*delta

	sgm = matrix(rep(3*sigma^2*pi*(pi-2)/N/2, s*s), s, s)

	f = c(x, (1-x)/3, (1-x)/3, (1-x)/3)

	for (i in 1:s)	sgm[i,i] = sgm[i,i]+3*sigma^2/N/f[i]/2

	lower = z.eta*sqrt(3*sigma^2*(1/f-2*pi+pi^2)/N/2)

	upper = rep(Inf,s)

	# unconditional power - set.pow

	as.numeric(pmvnorm(lower=lower, upper=upper, mean=mean, sigma=sgm)[1])-set.pow		
	}

## minimum required f_1 for uncondition power=set.pow

f1.un = uniroot(un.pow, c(0.05, 1/s), tol=0.0001, a=c(s, u, pi, delta, sigma, N, eta, set.pow))[1]

f1.un = round(as.numeric(f1.un), digits=2)

con.pow=function(x, a)	# x=f[1] a=c(s, u, pi, delta, sigma, N, eta, z.alpha, set.pow)
	{
	s = a[1]
	u = a[2:(s+1)]
	pi = a[s+2]
	delta = a[s+3]
	sigma = a[s+4]
	N = a[s+5]
	eta = a[s+6]
	z.alpha = a[s+7]
	set.pow = a[s+8]

	z.eta = qnorm(1-eta)

	mean = c((u-pi)*delta, delta)

	sgm = matrix(rep(0, (s+1)*(s+1)), (s+1), (s+1))

	sgm[1:s,1:s] = matrix(rep(3*sigma^2*pi*(pi-2)/N/2, s*s), s, s)

	f = c(x, (1-x)/3, (1-x)/3, (1-x)/3)

	for (i in 1:s)	sgm[i,i] = sgm[i,i]+3*sigma^2/N/f[i]/2

	sgm[(s+1),] = 3*sigma^2*(1-pi)/N/2

	sgm[,(s+1)] = 3*sigma^2*(1-pi)/N/2

	sgm[(s+1),(s+1)] = 3*sigma^2/N/2

	lower = c(z.eta*sqrt(3*sigma^2*(1/f-2*pi+pi^2)/N/2), z.alpha*sqrt(3*sigma^2/N/2))

	upper = rep(Inf, (s+1))

	# conditional power - set.pow

	as.numeric(pmvnorm(lower=lower, upper=upper, mean=mean, sigma=sgm)[1])/(1-pnorm(z.alpha-delta/sqrt(3*sigma^2/N/2)))-set.pow	
	}

## minimum required f_1 for condition power=set.pow

f1.con = uniroot(con.pow, c(0.05, 1/s), tol=0.0001, a=c(s, u, pi, delta, sigma, N, eta, z.alpha, set.pow))[1]

f1.con = round(as.numeric(f1.con), digits=2)


return(list(uncon.f1 = f1.un, con.f1 = f1.con))

}

###################

# f= c(x, x, 0.5-x, 0.5-x)

f12.def3 = function(alpha=0.025, beta=0.01, delta=0.005, sigma=0.013, u=c(1,1,1,1), pi=0, eta=0.4, set.pow=0.8)  

{
z.alpha = qnorm(1-alpha)

z.beta = qnorm(1-beta)

## Total number of patients for placebo group to detect overall treatment effect delta 
## with type I error=alpha, power=1-beta with 2 (trxt):1(placebo) allocation

N = 3*sigma^2*(z.alpha+z.beta)^2/2/delta^2	

N = round(N)

s = 4

un.pow=function(x, a)	# x=f[1] a=c(s, u, pi, delta, sigma, N, eta, set.pow)
	{
	s = a[1]
	u = a[2:(s+1)]
	pi = a[s+2]
	delta = a[s+3]
	sigma = a[s+4]
	N = a[s+5]
	eta = a[s+6]
	set.pow = a[s+7]

	z.eta = qnorm(1-eta)

	mean = (u-pi)*delta

	sgm = matrix(rep(3*sigma^2*pi*(pi-2)/N/2, s*s), s, s)

	f = c(x, x, 0.5-x, 0.5-x)

	for (i in 1:s)	sgm[i,i] = sgm[i,i]+3*sigma^2/N/f[i]/2

	lower = z.eta*sqrt(3*sigma^2*(1/f-2*pi+pi^2)/N/2)

	upper = rep(Inf,s)

	# unconditional power - set.pow

	as.numeric(pmvnorm(lower=lower, upper=upper, mean=mean, sigma=sgm)[1])-set.pow		
	}

## minimum required f_1 for uncondition power=set.pow

f1.un = uniroot(un.pow, c(0.05, 1/s), tol=0.0001, a=c(s, u, pi, delta, sigma, N, eta, set.pow))[1]

f1.un = round(as.numeric(f1.un), digits=2)

con.pow=function(x, a)	# x=f[1] a=c(s, u, pi, delta, sigma, N, eta, z.alpha, set.pow)
	{
	s = a[1]
	u = a[2:(s+1)]
	pi = a[s+2]
	delta = a[s+3]
	sigma = a[s+4]
	N = a[s+5]
	eta = a[s+6]
	z.alpha = a[s+7]
	set.pow = a[s+8]

	z.eta = qnorm(1-eta)

	mean = c((u-pi)*delta, delta)

	sgm = matrix(rep(0, (s+1)*(s+1)), (s+1), (s+1))

	sgm[1:s,1:s] = matrix(rep(3*sigma^2*pi*(pi-2)/N/2, s*s), s, s)

	f = c(x, x, 0.5-x, 0.5-x)

	for (i in 1:s)	sgm[i,i] = sgm[i,i]+3*sigma^2/N/f[i]/2

	sgm[(s+1),] = 3*sigma^2*(1-pi)/N/2

	sgm[,(s+1)] = 3*sigma^2*(1-pi)/N/2

	sgm[(s+1),(s+1)] = 3*sigma^2/N/2

	lower = c(z.eta*sqrt(3*sigma^2*(1/f-2*pi+pi^2)/N/2), z.alpha*sqrt(3*sigma^2/N/2))

	upper = rep(Inf, (s+1))

	# conditional power - set.pow

	as.numeric(pmvnorm(lower=lower, upper=upper, mean=mean, sigma=sgm)[1])/(1-pnorm(z.alpha-delta/sqrt(3*sigma^2/N/2)))-set.pow	
	}

## minimum required f_1 for condition power=set.pow

f1.con = uniroot(con.pow, c(0.05, 1/s), tol=0.0001, a=c(s, u, pi, delta, sigma, N, eta, z.alpha, set.pow))[1]

f1.con = round(as.numeric(f1.con), digits=2)


return(list(uncon.f1 = f1.un, con.f1 = f1.con))

}

###################

# f = c(x, x, x, 1-3*x)

f13.def3 = function(alpha=0.025, beta=0.01, delta=0.005, sigma=0.013, u=c(1,1,1,1), pi=0, eta=0.4, set.pow=0.8)  
{
z.alpha = qnorm(1-alpha)

z.beta = qnorm(1-beta)

## Total number of patients for placebo group to detect overall treatment effect delta 
## with type I error=alpha, power=1-beta with 2 (trxt):1(placebo) allocation

N = 3*sigma^2*(z.alpha+z.beta)^2/2/delta^2	

N = round(N)

s = 4

un.pow=function(x, a)	# x=f[1] a=c(s, u, pi, delta, sigma, N, eta, set.pow)
	{
	s = a[1]
	u = a[2:(s+1)]
	pi = a[s+2]
	delta = a[s+3]
	sigma = a[s+4]
	N = a[s+5]
	eta = a[s+6]
	set.pow = a[s+7]

	z.eta = qnorm(1-eta)

	mean = (u-pi)*delta

	sgm = matrix(rep(3*sigma^2*pi*(pi-2)/N/2, s*s), s, s)

	f = c(x, x, x, 1-3*x)

	for (i in 1:s)	sgm[i,i] = sgm[i,i]+3*sigma^2/N/f[i]/2

	lower = z.eta*sqrt(3*sigma^2*(1/f-2*pi+pi^2)/N/2)

	upper = rep(Inf,s)

	#unconditional power - set.pow

	as.numeric(pmvnorm(lower=lower, upper=upper, mean=mean, sigma=sgm)[1])-set.pow	
	}

## minimum required f_1 for uncondition power=set.pow

f1.un = uniroot(un.pow, c(0.05, 1/s), tol=0.0001, a=c(s, u, pi, delta, sigma, N, eta, set.pow))[1]

f1.un = round(as.numeric(f1.un), digits=2)

con.pow=function(x, a)	# x=f[1] a=c(s, u, pi, delta, sigma, N, eta, z.alpha, set.pow)
	{
	s = a[1]
	u = a[2:(s+1)]
	pi = a[s+2]
	delta = a[s+3]
	sigma = a[s+4]
	N = a[s+5]
	eta = a[s+6]
	z.alpha = a[s+7]
	set.pow = a[s+8]

	z.eta = qnorm(1-eta)

	mean = c((u-pi)*delta, delta)

	sgm = matrix(rep(0, (s+1)*(s+1)), (s+1), (s+1))

	sgm[1:s,1:s] = matrix(rep(3*sigma^2*pi*(pi-2)/N/2, s*s), s, s)

	f = c(x, x, x, 1-3*x)

	for (i in 1:s)	sgm[i,i] = sgm[i,i]+3*sigma^2/N/f[i]/2

	sgm[(s+1),] = 3*sigma^2*(1-pi)/N/2

	sgm[,(s+1)] = 3*sigma^2*(1-pi)/N/2

	sgm[(s+1),(s+1)] = 3*sigma^2/N/2

	lower = c(z.eta*sqrt(3*sigma^2*(1/f-2*pi+pi^2)/N/2), z.alpha*sqrt(3*sigma^2/N/2))

	upper = rep(Inf, (s+1))

	# conditional power - set.pow 

	as.numeric(pmvnorm(lower=lower, upper=upper, mean=mean, sigma=sgm)[1])/(1-pnorm(z.alpha-delta/sqrt(3*sigma^2/N/2)))-set.pow
	}

## minimum required f_1 for condition power=set.pow

f1.con = uniroot(con.pow, c(0.05, 1/s), tol=0.0001, a=c(s, u, pi, delta, sigma, N, eta, z.alpha, set.pow))[1]

f1.con = round(as.numeric(f1.con), digits=2)


return(list(uncon.f1 = f1.un, con.f1 = f1.con))

}



#### Specify the parameters. Default: alpha=0.025 beta=0.01 delta=0.005 sigma=0.013 u=c(1,1,1,1) 
##                                    pi=0 eta=0.4 set.pow=0.8

alpha = 0.025

beta = 0.01

delta = 0.005

sigma = 0.013

pi = 0

eta = 0.4

u=rep(1, 4)

set.pow = 0.8	# For some higher powers, there may not be a solution, may need to increase total sample size

f11.def3(alpha, beta, delta, sigma, u, pi, eta, set.pow)

f12.def3(alpha, beta, delta, sigma, u, pi, eta, set.pow)

f13.def3(alpha, beta, delta, sigma, u, pi, eta, set.pow)




